"""proxy fqdn

Revision ID: 27724e407e2b
Revises: 3c95509a5de9
Create Date: 2025-09-10 16:08:17.079028

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '27724e407e2b'
down_revision: Union[str, Sequence[str], None] = '3c95509a5de9'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('k8s_ext_control_plane', sa.Column('proxy_stack_fqdn', sa.String(length=253), nullable=True))
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('k8s_ext_control_plane', 'proxy_stack_fqdn')
    # ### end Alembic commands ###
